﻿#include <iostream>
#include <algorithm>
#include <unordered_set>
#include "MeshLoader.h"
#include "Hash.h"

const std::vector<Node>& MeshLoader::getNodes() {
    return Nodes;
}

const std::vector<FiniteElement>& MeshLoader::getFiniteElement() {
    return element;
}

const std::vector<FiniteElement>& MeshLoader::getBoundElement() {
    return boundelement;
}


std::vector<int> MeshLoader::getFiniteElementID(int _id1, int _id2, int _id3) {
    std::vector<int> res;
    auto po = [&_id1, &_id2, &_id3, &res](const FiniteElement& elem) {
        if ((std::find(elem.NodeID.begin(), elem.NodeID.end(), _id1) != elem.NodeID.end()) &&
            (std::find(elem.NodeID.begin(), elem.NodeID.end(), _id2) != elem.NodeID.end()) &&
            (std::find(elem.NodeID.begin(), elem.NodeID.end(), _id3) != elem.NodeID.end())) {
            res.push_back(elem.FiniteElementID);
        }
    };

    std::for_each(element.begin(), element.end(),po);
    return res;
}

std::vector<int> MeshLoader::getFiniteElementEdge(int _id1, int _id2) {
    std::vector<int> res;
    auto po = [&_id1, &_id2, &res](const FiniteElement& elem) {
        if ((std::find(elem.NodeID.begin(), elem.NodeID.end(), _id1) != elem.NodeID.end()) &&
            (std::find(elem.NodeID.begin(), elem.NodeID.end(), _id2) != elem.NodeID.end())) {
            res.push_back(elem.FiniteElementID);
        }
    };

    std::for_each(element.begin(), element.end(),po);
    return res;
}

std::vector<int> MeshLoader::getNodesId(int _id) {
    std::vector<int> res;
    auto cur = element.begin();

    auto po = [_id](const FiniteElement& elem) {
        return elem.GeometryAreaID == _id;
    };

    while (cur != element.end()) {
        cur = std::find_if(cur, element.end(), po);
        if (cur != element.end()) {
            for (auto node : cur->NodeID) {
                Node nodeId = Nodes[node];
                res.push_back(nodeId.NodeID);
            }
            ++cur;
        }
    }
    return std::vector<int>(res.begin(), res.end());
}

std::vector<int> MeshLoader::getFiniteElementIdSurface(int _id) {
    std::vector<int> res;
    auto po = [&_id, &res](const FiniteElement& elem) {
        if (elem.GeometryAreaID == _id) {
            res.push_back(elem.FiniteElementID);
        }
    };

    std::for_each(element.begin(), element.end(), po);
    return res;
}

std::vector<int> MeshLoader::getFiniteElementIDVol(int _id) {
    std::vector<int> res;
    auto po = [&_id, &res](const FiniteElement& elem) {
        if (elem.GeometryAreaID == _id) {
            res.push_back(elem.FiniteElementID);
        }
    };

    std::for_each(element.begin(), element.end(), po);
    return res;
}

void MeshLoader::printNode() {
    std::cout << Nodes;
}

void MeshLoader::printFiniteElement() {
    std::cout << element;
}

void MeshLoader::insertNodeMid() {
    std::unordered_set<Edge, hash> edge;
    for (auto& elem : element) {
        std::vector<int> ElemNideID = elem.NodeID;
        for (int first = 0; first < 4; ++first) {
            for (auto sec = first + 1; sec < 4; ++sec) {
                Edge curEdge(ElemNideID[first], ElemNideID[sec]);
                if (edge.insert(curEdge).second) {
                    Node newNode = getMiddleNode(curEdge);
                    curEdge.updMid(newNode.NodeID);
                    Nodes.push_back(newNode);
                    elem.NodeID.push_back(newNode.NodeID);
                }
                else {
                    elem.NodeID.push_back(curEdge.midNode);
                }
            }
        }
    }
    for (auto& belem : boundelement) {
        std::vector<int> bfeNodesId = belem.NodeID;
        for (auto first = 0; first < 3; ++first) {
            for (auto sec = first + 1; sec < 3; ++sec) {
                auto curEdgeIter = edge.find({ bfeNodesId[first], bfeNodesId[sec]});
                if (curEdgeIter == edge.end()) {
                    curEdgeIter = edge.find({ bfeNodesId[sec], bfeNodesId[first]});
                }
                belem.NodeID.push_back(curEdgeIter->midNode);
            }
        }
    }
}


std::vector<double> MeshLoader::getMiddleEdge(const Edge& edge) {
    double MiddleX = ((Nodes.at(edge.edgeNodes.first - 1).coordinate.at(0)) +
        (Nodes.at(edge.edgeNodes.second - 1).coordinate.at(0))) / 2;
    double MiddleY = ((Nodes.at(edge.edgeNodes.first - 1).coordinate.at(1)) +
        (Nodes.at(edge.edgeNodes.second - 1).coordinate.at(1))) / 2;
    double MiddleZ = ((Nodes.at(edge.edgeNodes.first - 1).coordinate.at(2)) +
        (Nodes.at(edge.edgeNodes.second - 1).coordinate.at(2))) / 2;
    return { MiddleX, MiddleY, MiddleZ };
}

Node MeshLoader::getMiddleNode(const Edge& edge) {
    Node newNode(Nodes.size() + 1, getMiddleEdge(edge), false);
    return newNode;
}

std::vector<double> MeshLoader::getCoordinate(int id) {
    return Nodes.at(id).coordinate;
}

std::vector<std::set<int>> MeshLoader::createNeighborsVector() {
    std::vector<std::set<int>> neighbors(Nodes.size() + 1);
    for (const auto& elem : boundelement) {
        for (auto nodeId : elem.NodeID)
            for (auto anthNodeId : elem.NodeID)
                if (nodeId != anthNodeId)
                    neighbors[nodeId].insert(anthNodeId);
    }

    return neighbors;
}








